# $Id: //LandSlide/24.3/tas/tcl/common.tcl#1 $
#
# Copyright (c) 2009 Spirent Communications, Inc.
# All rights reserved.
# CONFIDENTIAL AND PROPRIETARY
#
# This product is distributed under licenses restricting its use,
# copying, distribution, disclosure and decompilation. No part of this product may 
# be reproduced in any form by any means without prior written 
# authorization of Spirent Communications, Inc. and its licensors, if any.
#
################################################################################

variable MAX_MEM
#See if max memory is set in environment
if {![info exists MAX_MEM]} {
   set MAX_MEM 128m
}
if {[info exists env(JAVA_MAX_MEMORY)]} {
   set MAX_MEM $env(JAVA_MAX_MEMORY)
}
puts "Starting Java with Max Heap=$MAX_MEM"
set tclblend_init "-Xmx$MAX_MEM"
#set tclblend_init "-Xmx$MAX_MEM -Xshare:off"
#set tclblend_init "-Xcheck:jni -verbose:jni,class -Xmx$MAX_MEM"
package require java

java::import com.spirent.ls.tcl.TestSession
java::import com.spirent.ls.tcl.TsApi
java::import com.spirent.ls.tcl.Client

namespace eval ::ls {

variable TRUE true
variable FALSE false

variable LOG_DEBUG 0
variable LOG_INFO 1
variable LOG_MINOR 2
variable LOG_MAJOR 3
variable LOG_WARNING 4
variable LOG_ERROR 5
variable LOG_IMPORTANT 6
variable LOG_SILENT 100

variable ETH_DUPLEX_NONE ""
variable ETH_DUPLEX_HALF Half
variable ETH_DUPLEX_FULL Full

variable ETH_SPEED_NONE ""
variable ETH_SPEED_10 10
variable ETH_SPEED_100 100
variable ETH_SPEED_1000 1000

variable STEP_BLANK -1
variable STEP_LOOP_START -2
variable STEP_LOOP_END -3
variable STEP_WAIT -4

variable STEP_STATE_NONE ""
variable STEP_STATE_RUNNING Running
variable STEP_STATE_STOPPED Stopped

variable STEP_ACTIVITY_NONE ""
variable STEP_ACTIVITY_INIT Init
variable STEP_ACTIVITY_START Start
variable STEP_ACTIVITY_STOP Stop
variable STEP_ACTIVITY_CLEANUP Cleanup

variable DMF_MSG_TYPE_NONE 0
variable DMF_MSG_TYPE_REQUEST 1
variable DMF_MSG_TYPE_RESPONSE 2

variable REPORT_SUMMARY -1
variable REPORT_ALL -2
variable REPORT_EVERYTHING -3

variable FORMAT_XLS 0
variable FORMAT_CSV 1

variable TEST_NODE_TYPE_ATM atm
variable TEST_NODE_TYPE_ETH eth

variable VSA_TYPE_VSA VSA
variable VSA_TYPE_AVP AVP

variable stdoutlogging ""
variable stdoutfile ""
variable tas

java::load com.spirent.ls.tcl.LandslideExtensions

if {![info exists tas]} {
   set tas [java::new com.spirent.ls.tcl.Client]
   set tas [java::null]
}

proc retrieve { args } {
  return [eval ls_retrieve [lrange $args 0 end]]
}

proc get { args } {

  if { [llength $args] >= 2 } {
    set key [string tolower [lindex $args 1]]
    if { "-query" == $key } {
       return [eval ls_query measurements [lindex $args 0] [lrange $args 2 end]]
    }
    if { "-measurement" == $key } {
       return [eval ls_query measurements [lindex $args 0] [lrange $args 2 end]]
    }
  }
  return [eval ls_get [lrange $args 0 end]]
}

proc config { args } {
  return [eval ls_config [lrange $args 0 end]]
}

proc save { args } {
  return [eval ls_save [lrange $args 0 end]]
}

proc create { args } {
  return [eval ls_create [lrange $args 0 end]]
}

proc delete { args } {
  return [eval ls_delete [lrange $args 0 end]]
}

proc perform { args } {
  return [eval ls_perform [lrange $args 0 end]]
}

proc disconnect { args } {
  return [eval ls_disconnect [lrange $args 0 end]]
}

proc reconnect { args } {
  return [eval ls_reconnect [lrange $args 0 end]]
}

proc query { args } {
   return [eval ls_query [lrange $args 0 end]]
}

proc register { args } {
   return [eval ls_register [lrange $args 0 end]]
}

proc deregister { args } {
   return [eval ls_deregister [lrange $args 0 end]]
}

proc log { args } {
   variable LOG_IMPORTANT 
   if { [llength $args] > 1 } {
     java::call com.sseworks.sp.common.TclUtil Log "[lindex $args 0]" "[lindex $args 1]"
   } else {
     java::call com.sseworks.sp.common.TclUtil Log $LOG_IMPORTANT "[lindex $args 0]" 
   }
}

proc help { args } {
   return [eval ls_help [lrange $args 0 end]]
}

proc setStdOutLoggingOn { } {
  config ApiOptions -stdOutLogging true
}

proc setStdOutLoggingOff { } {
  config ApiOptions -stdOutLogging false
}

proc setFileLoggingOn { file } {
  config ApiOptions -fileLogging $file
}

proc setFileLoggingOff { } {
  config ApiOptions -fileLogging ""
}

proc print { msg } {
  java::call com.sseworks.sp.common.TclUtil Log "$msg"
}

proc sleep { seconds } {
  after [expr {int($seconds * 1000)}]
}

proc convertToStringArray args {
  set newCall "java::new {String\[\]} {[llength $args]}"
  set ret [eval $newCall]
  set i 0
  foreach arg $args {
     $ret set $i $arg
     set i [expr $i + 1]
  }
  return $ret
}

proc login { host_ip username password { is_secure "false" } { port 0 } } {
  variable tas
  variable err ""
  java::try {

    if { [java::isnull $tas] } {
	
		set pattern "(true)|(false)"
		if {[regexp $pattern $is_secure match]} {
		  ##OK
		} else {
			set err "Login failed: Invalid is_secure argument, expected true or false"
		    return -code error "Invalid is_secure argument, expected true or false"
		}
		
		if {[string match {*[0-9]*} $port]} { 
		   ##OK
		} else {
			set err "Login failed: Invalid port argument, expected 0 - 65535"
		    return -code error "Invalid port argument, expected 0 - 65535"
		}

      set tas [java::new com.spirent.ls.tcl.Client]
	    java::try {
	     config $tas -IsSecure $is_secure 
	  } catch { Throwable e} {
		$e printStackTrace
	    set tas [java::null]
		return -code error "Invalid is_secure argument, expected true or false: [$e toString]"
	  }
      print "logging into $host_ip with port $port"

      set rsp [$tas -noconvert login $username $password $host_ip $port ]
      set warnings [$tas -noconvert getWarnings]

      if { [$warnings length] > 0 } {
        print "Warnings: [$warnings toString]"
      }

      if { [java::isnull $rsp] } {
         print "Login successful to $host_ip as $username!!"
         return $tas
      } else {
         print "Login failed: [$rsp toString]"
         $tas logout
         set tas [java::null]
         set err "Login failed: [$rsp toString]"
         return -code error "Login failed: [$rsp toString]"
      }

    } else {
       print "You must first logout existing connection"
	 set err "You must first logout existing connection"
       return -code error "You must first logout existing connection"
    }
  } catch {TclException err} {
    return -code error "TclException in procedure: [$err toString]"
  } catch { Exception e} {
    return -code error "error in login: [$e toString]"
  } catch { NoClassDefFoundError e} {
    return -code error "Classpath not set properly: [$e toString]"
  } catch { Throwable e} {
    $e printStackTrace
    return -code error "Throwable: [$e toString]"
  }
  if { $err == "" && ! [java::isnull $tas] } {
    return $tas 
  } else {
    return -code error $err
  }
}

proc logout { } {
  variable tas

  if { [java::isnull $tas] == 0 } {
    $tas logout
    set tas [java::null]
  }
  return
}

proc isConnected { } {
  variable tas

  if { [java::isnull $tas] } {
    return 0
  } else {
    return [$tas isConnected]
  }
}

proc dumpClientLog { } {
  variable tas
  if { [java::isnull $tas] == 0 } {
    $tas printClientLogs
  }
}

proc showLastCommandDetails { } {
  variable tas
  if { [java::isnull $tas] == 0 } {
    java::call com.spirent.ls.tcl.TclLoggedAction PrintLastCommand
  }
}

proc determineChildrenListSize { child_name list } {
  set len [llength $list]
  ## If it's 2 elements and the first item is not a list, i.e. the first item is the name: NAME VALUE
  if { $len == 2 && [llength [lindex $list 0]] == 1 } {
    set len 0
    if { [string first $child_name [lindex $list 0]] == 0 } {
      set len 1
    }
  }  
  return $len
}

proc alwaysReturnList { child_name list } {
  set len [llength $list]
  ## If it's 2 elements and the first item is not a list, i.e. the first item is the name: NAME VALUE
  if { $len == 2 &&  [llength [lindex $list 0]] == 1 } {
    if { [string first $child_name [lindex $list 0]] == 0 } {
      return [list $list] 
    }
  }  
  return $list
}


proc CRS {args} {
  upvar evalResult evalResult
  upvar cmdReturn cmdReturn
  variable ::errorInfo
  set status 1
  set cmd "$args"
  if {[catch {eval $cmd} cmdReturn] } {
    if {([regexp "LSERROR" $errorInfo] == 1) || ([regexp "lserror" $errorInfo] == 1)}
    {
      puts "Command $cmd failed!"
      puts "Error text is $errorInfo"
      exit
    } else {
      #Tcl Crash
      puts "Command crashed. TCL error message was:"
      puts $cmdReturn
      puts $errorInfo
      exit
    }
  } else {
    set evalResult $cmdReturn
  } ;# end if-else statement
  return $evalResult
} ;# end procedure

}
